package tech.riemann.ims.controller.platform.acl;

import club.zhcs.lina.auth.encode.PasswordUtils;
import club.zhcs.lina.auth.service.AuthUser.Sex;
import club.zhcs.lina.starter.exception.BizException;
import club.zhcs.lina.utils.enums.Codebook;
import club.zhcs.lina.utils.enums.ICodeBook;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.RequiredArgsConstructor;
import org.nutz.lang.Strings;
import org.springframework.http.HttpStatus;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import tech.riemann.ims.dto.response.PermissionInfo;
import tech.riemann.ims.dto.response.RoleInfo;
import tech.riemann.ims.entity.acl.Permission;
import tech.riemann.ims.entity.acl.User;
import tech.riemann.ims.service.acl.IUserService;

import java.util.Arrays;
import java.util.List;

/**
 * <p>
 * 用户 前端控制器
 * </p>
 *
 * @author Kerbores(kerbores@gmail.com)
 *
 * @since 2024-07-22 13:56:54
 */
@RestController
@Tag(name = "User", description = "用户")
@RequiredArgsConstructor
public class UserController {

    private final IUserService userService;

    @GetMapping("user-all")
    @Operation(summary = "所有用户")
    public List<User> all() {
        return userService.list();
    }

    @GetMapping("users")
    @Operation(summary = "分页查询用户")
    public IPage<User> users(
                             @Parameter(description = "页码") @RequestParam(required = false, defaultValue = "1") int page,
                             @Parameter(description = "页面大小") @RequestParam(required = false, defaultValue = "10") int size,
                             @Parameter(description = "性别") @RequestParam(required = false, defaultValue = "") Sex sex,
                             @Parameter(description = "搜索关键词") @RequestParam(required = false, defaultValue = "") String key) {
        boolean hasLike = Strings.isNotBlank(key);
        key = String.format("%%%s%%", key);
        return userService.page(Page.<User> of(page, size),
                                Wrappers.<User> lambdaQuery()
                                        .like(hasLike, User::getName, key)
                                        .eq(sex != null, User::getSex, sex)
                                        .orderByDesc(User::getId));
    }

    @GetMapping("user/sexes")
    @Operation(summary = "用户性别")
    public List<Codebook> sexes() {
        return Arrays.stream(Sex.values()).map(ICodeBook::build).toList();
    }

    @PatchMapping("user/{name}/password")
    @Operation(summary = "重置密码")
    public String resetPassword(@Parameter(description = "用户名", required = true) @PathVariable String name) {
        return userService.resetPassword(name);
    }

    @GetMapping("user/{id}")
    @Operation(summary = "用户详情")
    public User detail(@Parameter(description = "用户id", required = true) @PathVariable long id) {
        return userService.getById(id);
    }

    @PutMapping("user")
    @Operation(summary = "增加/编辑用户")
    public User saveOrUpdateUser(@Validated @Parameter(description = "用户") @RequestBody User user) {
        if (user.getId() == null || user.getId() <= 0) {
            user.setPassword(PasswordUtils.randomSaltEncode(user.getPassword()));
            if (userService.save(user)) {
                return user;
            } else {
                throw BizException.create("保存用户失败!");
            }
        } else {
            if (userService.update(user, Wrappers.<User> lambdaUpdate().eq(User::getId, user.getId()))) {
                return user;
            } else {
                throw BizException.create("更新用户失败!");
            }
        }
    }

    @DeleteMapping("user/{id}")
    @Operation(summary = "删除用户")
    @ResponseStatus(HttpStatus.OK)
    public void deleteUser(@Parameter(description = "用户id", required = true) @PathVariable long id) {
        if (!userService.removeById(id)) {
            throw BizException.create("删除用户失败!");
        }
    }

    @GetMapping("user/{name}/permissions")
    @Operation(summary = "查询用户权限")
    public List<Permission> permissions(
                                        @Parameter(description = "用户名") @PathVariable String name) {
        return userService.permissionsByUserName(name);
    }

    @GetMapping("user/{name}/permission-infos")
    @Operation(summary = "查询用于授权的权限信息")
    public List<PermissionInfo> permissionInfos(
                                                @Parameter(description = "用户名") @PathVariable String name) {
        return userService.permissionInfosByUserName(name);
    }

    @PostMapping("user/{name}/permissions")
    @Operation(summary = "为指定用户授权")
    @ResponseStatus(HttpStatus.OK)
    public void grant(
                      @Parameter(description = "用户名") @PathVariable String name,
                      @Parameter(name = "permissions",
                              description = "待授权信息[keyPath]") @RequestBody List<String> permissions) {
        if (!userService.grant(name, permissions)) {
            throw BizException.create("授权失败");
        }
    }

    @GetMapping("user/{name}/role-infos")
    @Operation(summary = "查询用户权限")
    public List<RoleInfo> roleInfos(
                                    @Parameter(description = "用户名") @PathVariable String name) {
        return userService.roleInfosByUserName(name);
    }

    @PostMapping("user/{name}/roles")
    @Operation(summary = "为指定用户设置角色")
    @ResponseStatus(HttpStatus.OK)
    public void grantRole(
                          @Parameter(description = "用户名") @PathVariable String name,
                          @Parameter(name = "roles", description = "待授权角色") @RequestBody List<String> roles) {
        if (!userService.grantRole(name, roles)) {
            throw BizException.create("授权角色失败");
        }
    }

}
